@@ -5,11 +5,10 @@ import (
 	"errors"
 	"fmt"
 	"reflect"
-	"strings"
 	"time"
 
 	"github.com/clarketm/json"
-	igntypes "github.com/coreos/ignition/config/v2_2/types"
+	ign3types "github.com/coreos/ignition/v2/config/v3_1/types"
 	"github.com/golang/glog"
 	"github.com/imdario/mergo"
 	"github.com/vincent-petithory/dataurl"
@@ -327,7 +326,7 @@ func (ctrl *Controller) handleFeatureErr(err error, key interface{}) {
 	ctrl.featureQueue.AddAfter(key, 1*time.Minute)
 }
 
-func (ctrl *Controller) generateOriginalKubeletConfig(role string) (*igntypes.File, error) {
+func (ctrl *Controller) generateOriginalKubeletConfig(role string) (*ign3types.File, error) {
 	cc, err := ctrl.ccLister.Get(ctrlcommon.ControllerConfigName)
 	if err != nil {
 		return nil, fmt.Errorf("could not get ControllerConfig %v", err)
@@ -440,7 +439,10 @@ func (ctrl *Controller) syncKubeletConfig(key string) error {
 	for _, pool := range mcpPools {
 		role := pool.Name
 		// Get MachineConfig
-		managedKey := getManagedKubeletConfigKey(pool)
+		managedKey, err := getManagedKubeletConfigKey(pool, ctrl.client)
+		if err != nil {
+			return err
+		}
 		mc, err := ctrl.client.MachineconfigurationV1().MachineConfigs().Get(context.TODO(), managedKey, metav1.GetOptions{})
 		if err != nil && !macherrors.IsNotFound(err) {
 			return ctrl.syncStatusOnly(cfg, err, "could not find MachineConfig: %v", managedKey)
@@ -451,7 +453,7 @@ func (ctrl *Controller) syncKubeletConfig(key string) error {
 		if err != nil {
 			return ctrl.syncStatusOnly(cfg, err, "could not generate the original Kubelet config: %v", err)
 		}
-		dataURL, err := dataurl.DecodeString(originalKubeletIgn.Contents.Source)
+		dataURL, err := dataurl.DecodeString(*originalKubeletIgn.Contents.Source)
 		if err != nil {
 			return ctrl.syncStatusOnly(cfg, err, "could not decode the original Kubelet source string: %v", err)
 		}
@@ -480,7 +482,7 @@ func (ctrl *Controller) syncKubeletConfig(key string) error {
 		}
 		if isNotFound {
 			ignConfig := ctrlcommon.NewIgnConfig()
-			mc, err = mtmpl.MachineConfigFromIgnConfig(role, managedKey, ignConfig)
+			mc, err = ctrlcommon.MachineConfigFromIgnConfig(role, managedKey, ignConfig)
 			if err != nil {
 				return ctrl.syncStatusOnly(cfg, err, "could not create MachineConfig from new Ignition config: %v", err)
 			}
@@ -572,24 +574,17 @@ func (ctrl *Controller) addFinalizerToKubeletConfig(kc *mcfgv1.KubeletConfig, mc
 			return err
 		}
 
-		kcTmp := newcfg.DeepCopy()
-		// We want to use the mc name as the finalizer instead of the uid because
-		// every time a resync happens, a new uid is generated. This is why the list
-		// of finalizers had multiple entries. So check if the list of finalizers consists
-		// of uids, if it does then clear the list of finalizers and we will add the mc
-		// name to it, ensuring we don't have duplicate or multiple finalizers.
+		uid := string(mc.ObjectMeta.GetUID())
+
+		// if the finalizer is already set then skip
 		for _, finalizerName := range newcfg.Finalizers {
-			if !strings.Contains(finalizerName, "kubelet") {
-				kcTmp.ObjectMeta.SetFinalizers([]string{})
+			if finalizerName == mc.Name || finalizerName == uid {
+				return nil
 			}
 		}
-		// Only append the mc name if it is not already in the list of finalizers.
-		// When we update an existing kubeletconfig, the generation number increases causing
-		// a resync to happen. When this happens, the mc name is the same, so we don't
-		// want to add duplicate entries to the list of finalizers.
-		if !ctrlcommon.InSlice(mc.Name, kcTmp.ObjectMeta.Finalizers) {
-			kcTmp.ObjectMeta.Finalizers = append(kcTmp.ObjectMeta.Finalizers, mc.Name)
-		}
+
+		kcTmp := newcfg.DeepCopy()
+		kcTmp.ObjectMeta.Finalizers = append(kcTmp.ObjectMeta.Finalizers, uid)
 
 		modJSON, err := json.Marshal(kcTmp)
 		if err != nil {
