@@ -7,23 +7,21 @@ import (
 	"testing"
 	"time"
 
-	ign "github.com/coreos/ignition/config/v2_2"
-	igntypes "github.com/coreos/ignition/config/v2_2/types"
-	ign3types "github.com/coreos/ignition/v2/config/v3_0/types"
+	ign3types "github.com/coreos/ignition/v2/config/v3_1/types"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+	corev1 "k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/labels"
+	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/util/uuid"
+	"k8s.io/apimachinery/pkg/util/wait"
+
 	mcfgv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 	ctrlcommon "github.com/openshift/machine-config-operator/pkg/controller/common"
 	"github.com/openshift/machine-config-operator/pkg/daemon/constants"
 	"github.com/openshift/machine-config-operator/test/e2e/framework"
 	"github.com/openshift/machine-config-operator/test/helpers"
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
-	corev1 "k8s.io/api/core/v1"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/fields"
-	"k8s.io/apimachinery/pkg/labels"
-	"k8s.io/apimachinery/pkg/runtime"
-	"k8s.io/apimachinery/pkg/util/uuid"
-	"k8s.io/apimachinery/pkg/util/wait"
 )
 
 // Test case for https://github.com/openshift/machine-config-operator/issues/358
@@ -50,56 +48,13 @@ func TestMCDToken(t *testing.T) {
 	}
 }
 
-func mcLabelForRole(role string) map[string]string {
-	mcLabels := make(map[string]string)
-	mcLabels[mcfgv1.MachineConfigRoleLabelKey] = role
-	return mcLabels
-}
-
-func mcLabelForWorkers() map[string]string {
-	return mcLabelForRole("worker")
-}
-
-func createIgnFile(path, content, fs string, mode int) igntypes.File {
-	return igntypes.File{
-		FileEmbedded1: igntypes.FileEmbedded1{
-			Contents: igntypes.FileContents{
-				Source: content,
-			},
-			Mode: &mode,
-		},
-		Node: igntypes.Node{
-			Filesystem: fs,
-			Path:       path,
-			User: &igntypes.NodeUser{
-				Name: "root",
-			},
-		},
-	}
-}
-
-func createMCToAddFileForRole(name, role, filename, data, fs string) *mcfgv1.MachineConfig {
-	mcadd := createMC(fmt.Sprintf("%s-%s", name, uuid.NewUUID()), role)
-
-	ignConfig := ctrlcommon.NewIgnConfig()
-	ignFile := createIgnFile(filename, "data:,"+data, fs, 420)
-	ignConfig.Storage.Files = append(ignConfig.Storage.Files, ignFile)
-	rawIgnConfig := helpers.MarshalOrDie(ignConfig)
-	mcadd.Spec.Config.Raw = rawIgnConfig
-	return mcadd
-}
-
-func createMCToAddFile(name, filename, data, fs string) *mcfgv1.MachineConfig {
-	return createMCToAddFileForRole(name, "worker", filename, data, fs)
-}
-
 func TestMCDeployed(t *testing.T) {
 	cs := framework.NewClientSet("")
 
 	// TODO: bring this back to 10
 	for i := 0; i < 3; i++ {
 		startTime := time.Now()
-		mcadd := createMCToAddFile("add-a-file", fmt.Sprintf("/etc/mytestconf%d", i), "test", "root")
+		mcadd := createMCToAddFile("add-a-file", fmt.Sprintf("/etc/mytestconf%d", i), "test")
 
 		// create the dummy MC now
 		_, err := cs.MachineConfigs().Create(context.TODO(), mcadd, metav1.CreateOptions{})
@@ -122,123 +77,94 @@ func TestMCDeployed(t *testing.T) {
 	}
 }
 
-func mcdForNode(cs *framework.ClientSet, node *corev1.Node) (*corev1.Pod, error) {
-	// find the MCD pod that has spec.nodeNAME = node.Name and get its name:
-	listOptions := metav1.ListOptions{
-		FieldSelector: fields.SelectorFromSet(fields.Set{"spec.nodeName": node.Name}).String(),
-	}
-	listOptions.LabelSelector = labels.SelectorFromSet(labels.Set{"k8s-app": "machine-config-daemon"}).String()
-
-	mcdList, err := cs.Pods("openshift-machine-config-operator").List(context.TODO(), listOptions)
-	if err != nil {
-		return nil, err
-	}
-	if len(mcdList.Items) != 1 {
-		if len(mcdList.Items) == 0 {
-			return nil, fmt.Errorf("Failed to find MCD for node %s", node.Name)
-		}
-		return nil, fmt.Errorf("Too many (%d) MCDs for node %s", len(mcdList.Items), node.Name)
-	}
-	return &mcdList.Items[0], nil
-}
-
-func TestUpdateSSH(t *testing.T) {
-	cs := framework.NewClientSet("")
-
-	// create a dummy MC with an sshKey for user Core
-	mcName := fmt.Sprintf("00-sshkeys-worker-%s", uuid.NewUUID())
-	mcadd := &mcfgv1.MachineConfig{}
-	mcadd.ObjectMeta = metav1.ObjectMeta{
-		Name:   mcName,
-		Labels: mcLabelForWorkers(),
-	}
-	// create a new MC that adds a valid user & ssh keys
-	tempUser := igntypes.PasswdUser{
-		Name: "core",
-		SSHAuthorizedKeys: []igntypes.SSHAuthorizedKey{
-			"1234_test",
-			"abc_test",
-		},
-	}
-	ignConfig := ctrlcommon.NewIgnConfig()
-	ignConfig.Passwd.Users = append(ignConfig.Passwd.Users, tempUser)
-	rawIgnConfig := helpers.MarshalOrDie(ignConfig)
-	mcadd.Spec.Config.Raw = rawIgnConfig
-
-	_, err := cs.MachineConfigs().Create(context.TODO(), mcadd, metav1.CreateOptions{})
-	require.Nil(t, err, "failed to create MC")
-	t.Logf("Created %s", mcadd.Name)
-
-	// grab the latest worker- MC
-	renderedConfig, err := waitForRenderedConfig(t, cs, "worker", mcadd.Name)
-	require.Nil(t, err)
-	err = waitForPoolComplete(t, cs, "worker", renderedConfig)
-	require.Nil(t, err)
-	nodes, err := getNodesByRole(cs, "worker")
-	require.Nil(t, err)
-	for _, node := range nodes {
-		assert.Equal(t, node.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
-		assert.Equal(t, node.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
-
-		// now rsh into that daemon and grep the authorized key file to check if 1234_test was written
-		// must do both commands in same shell, combine commands into one exec.Command()
-		found := execCmdOnNode(t, cs, node, "grep", "1234_test", "/rootfs/home/core/.ssh/authorized_keys")
-		if !strings.Contains(found, "1234_test") {
-			t.Fatalf("updated ssh keys not found in authorized_keys, got %s", found)
-		}
-		t.Logf("Node %s has SSH key", node.Name)
-	}
-}
-
 func TestKernelArguments(t *testing.T) {
 	cs := framework.NewClientSet("")
+
+	// Create infra pool to roll out MC changes
+	unlabelFunc := labelRandomNodeFromPool(t, cs, "worker", "node-role.kubernetes.io/infra")
+	createMCP(t, cs, "infra")
+
+	// create old mc to have something to verify we successfully rolled back
+	oldInfraConfig := createMC("old-infra", "infra")
+	_, err := cs.MachineConfigs().Create(context.TODO(), oldInfraConfig, metav1.CreateOptions{})
+	require.Nil(t, err)
+	oldInfraRenderedConfig, err := waitForRenderedConfig(t, cs, "infra", oldInfraConfig.Name)
+	err = waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig)
+	require.Nil(t, err)
+
+	// create kargs MC
 	kargsMC := &mcfgv1.MachineConfig{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:   fmt.Sprintf("kargs-%s", uuid.NewUUID()),
-			Labels: mcLabelForWorkers(),
+			Labels: mcLabelForRole("infra"),
 		},
 		Spec: mcfgv1.MachineConfigSpec{
 			Config: runtime.RawExtension{
 				Raw: helpers.MarshalOrDie(ctrlcommon.NewIgnConfig()),
 			},
-			KernelArguments: []string{"nosmt", "foo=bar"},
+			KernelArguments: []string{"nosmt", "foo=bar", "foo=baz", " baz=test bar=hello world"},
 		},
 	}
 
-	_, err := cs.MachineConfigs().Create(context.TODO(), kargsMC, metav1.CreateOptions{})
+	_, err = cs.MachineConfigs().Create(context.TODO(), kargsMC, metav1.CreateOptions{})
 	require.Nil(t, err)
 	t.Logf("Created %s", kargsMC.Name)
-	renderedConfig, err := waitForRenderedConfig(t, cs, "worker", kargsMC.Name)
+	renderedConfig, err := waitForRenderedConfig(t, cs, "infra", kargsMC.Name)
 	require.Nil(t, err)
-	if err := waitForPoolComplete(t, cs, "worker", renderedConfig); err != nil {
-		t.Fatal(err)
-	}
-	nodes, err := getNodesByRole(cs, "worker")
+	err = waitForPoolComplete(t, cs, "infra", renderedConfig)
 	require.Nil(t, err)
-	for _, node := range nodes {
-		assert.Equal(t, node.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
-		assert.Equal(t, node.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
-		kargs := execCmdOnNode(t, cs, node, "cat", "/rootfs/proc/cmdline")
-		for _, v := range kargsMC.Spec.KernelArguments {
-			if !strings.Contains(kargs, v) {
-				t.Fatalf("Missing '%s' in kargs", v)
-			}
+
+	// Re-fetch the infra node for updated annotations
+	infraNode := getSingleNodeByRole(t, cs, "infra")
+	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
+	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+	kargs := execCmdOnNode(t, cs, infraNode, "cat", "/rootfs/proc/cmdline")
+	expectedKernelArgs := []string{"nosmt", "foo=bar", "foo=baz", "baz=test", "bar=hello world"}
+	for _, v := range expectedKernelArgs {
+		if !strings.Contains(kargs, v) {
+			t.Fatalf("Missing %q in kargs: %q", v, kargs)
 		}
-		t.Logf("Node %s has expected kargs", node.Name)
 	}
+	t.Logf("Node %s has expected kargs", infraNode.Name)
+
+	// cleanup - delete karg mc and rollback
+	if err := cs.MachineConfigs().Delete(context.TODO(), kargsMC.Name, metav1.DeleteOptions{}); err != nil {
+		t.Error(err)
+	}
+	t.Logf("Deleted MachineConfig %s", kargsMC.Name)
+	err = waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig)
+	require.Nil(t, err)
+
+	unlabelFunc()
+
+	workerMCP, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
+	require.Nil(t, err)
+	if err := wait.Poll(2*time.Second, 5*time.Minute, func() (bool, error) {
+		node, err := cs.Nodes().Get(context.TODO(), infraNode.Name, metav1.GetOptions{})
+		require.Nil(t, err)
+		if node.Annotations[constants.DesiredMachineConfigAnnotationKey] != workerMCP.Spec.Configuration.Name {
+			return false, nil
+		}
+		return true, nil
+	}); err != nil {
+		t.Errorf("infra node hasn't moved back to worker config: %v", err)
+	}
+	err = waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig)
+	require.Nil(t, err)
 }
 
 func TestKernelType(t *testing.T) {
 	cs := framework.NewClientSet("")
-	// Get initial MachineConfig used by the worker pool so that we can rollback to it later on
-	mcp, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
-	require.Nil(t, err)
-	workerOldMC := mcp.Status.Configuration.Name
 
+	unlabelFunc := labelRandomNodeFromPool(t, cs, "worker", "node-role.kubernetes.io/infra")
+
+	oldInfraRenderedConfig := getMcName(t, cs, "infra")
+
+	// create kernel type MC and roll out
 	kernelType := &mcfgv1.MachineConfig{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:   fmt.Sprintf("kerneltype-%s", uuid.NewUUID()),
-			Labels: mcLabelForWorkers(),
+			Labels: mcLabelForRole("infra"),
 		},
 		Spec: mcfgv1.MachineConfigSpec{
 			Config: runtime.RawExtension{
@@ -248,26 +174,24 @@ func TestKernelType(t *testing.T) {
 		},
 	}
 
-	_, err = cs.MachineConfigs().Create(context.TODO(), kernelType, metav1.CreateOptions{})
+	_, err := cs.MachineConfigs().Create(context.TODO(), kernelType, metav1.CreateOptions{})
 	require.Nil(t, err)
 	t.Logf("Created %s", kernelType.Name)
-	renderedConfig, err := waitForRenderedConfig(t, cs, "worker", kernelType.Name)
+	renderedConfig, err := waitForRenderedConfig(t, cs, "infra", kernelType.Name)
 	require.Nil(t, err)
-	if err := waitForPoolComplete(t, cs, "worker", renderedConfig); err != nil {
+	if err := waitForPoolComplete(t, cs, "infra", renderedConfig); err != nil {
 		t.Fatal(err)
 	}
-	nodes, err := getNodesByRole(cs, "worker")
-	require.Nil(t, err)
-	for _, node := range nodes {
-		assert.Equal(t, node.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
-		assert.Equal(t, node.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+	infraNode := getSingleNodeByRole(t, cs, "infra")
 
-		kernelInfo := execCmdOnNode(t, cs, node, "uname", "-a")
-		if !strings.Contains(kernelInfo, "PREEMPT RT") {
-			t.Fatalf("Node %s doesn't have expected kernel", node.Name)
-		}
-		t.Logf("Node %s has expected kernel", node.Name)
+	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
+	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+
+	kernelInfo := execCmdOnNode(t, cs, infraNode, "chroot", "/rootfs", "rpm", "-qa", "kernel-rt-core")
+	if !strings.Contains(kernelInfo, "kernel-rt-core") {
+		t.Fatalf("Node %s doesn't have expected kernel", infraNode.Name)
 	}
+	t.Logf("Node %s has expected kernel", infraNode.Name)
 
 	// Delete the applied kerneltype MachineConfig to make sure rollback works fine
 	if err := cs.MachineConfigs().Delete(context.TODO(), kernelType.Name, metav1.DeleteOptions{}); err != nil {
@@ -277,30 +201,126 @@ func TestKernelType(t *testing.T) {
 	t.Logf("Deleted MachineConfig %s", kernelType.Name)
 
 	// Wait for the mcp to rollback to previous config
-	if err := waitForPoolComplete(t, cs, "worker", workerOldMC); err != nil {
+	if err := waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig); err != nil {
 		t.Fatal(err)
 	}
 
-	// Re-fetch the worker nodes for updated annotations
-	nodes, err = getNodesByRole(cs, "worker")
+	// Re-fetch the infra node for updated annotations
+	infraNode = getSingleNodeByRole(t, cs, "infra")
+
+	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], oldInfraRenderedConfig)
+	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+	kernelInfo = execCmdOnNode(t, cs, infraNode, "chroot", "/rootfs", "rpm", "-qa", "kernel-rt-core")
+	if strings.Contains(kernelInfo, "kernel-rt-core") {
+		t.Fatalf("Node %s did not rollback successfully", infraNode.Name)
+	}
+	t.Logf("Node %s has successfully rolled back", infraNode.Name)
+
+	unlabelFunc()
+
+	workerMCP, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
 	require.Nil(t, err)
-	for _, node := range nodes {
-		assert.Equal(t, node.Annotations[constants.CurrentMachineConfigAnnotationKey], workerOldMC)
-		assert.Equal(t, node.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
-		kernelInfo := execCmdOnNode(t, cs, node, "uname", "-a")
-		if strings.Contains(kernelInfo, "PREEMPT RT") {
-			t.Fatalf("Node %s did not rollback successfully", node.Name)
+	if err := wait.Poll(2*time.Second, 5*time.Minute, func() (bool, error) {
+		node, err := cs.Nodes().Get(context.TODO(), infraNode.Name, metav1.GetOptions{})
+		require.Nil(t, err)
+		if node.Annotations[constants.DesiredMachineConfigAnnotationKey] != workerMCP.Spec.Configuration.Name {
+			return false, nil
 		}
-		t.Logf("Node %s has successfully rolled back", node.Name)
+		return true, nil
+	}); err != nil {
+		t.Errorf("infra node hasn't moved back to worker config: %v", err)
+	}
+	err = waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig)
+	require.Nil(t, err)
+
+}
+
+func TestExtensions(t *testing.T) {
+	cs := framework.NewClientSet("")
+
+	unlabelFunc := labelRandomNodeFromPool(t, cs, "worker", "node-role.kubernetes.io/infra")
+
+	oldInfraRenderedConfig := getMcName(t, cs, "infra")
+
+	// Apply extensions
+	extensions := &mcfgv1.MachineConfig{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:   fmt.Sprintf("extensions-%s", uuid.NewUUID()),
+			Labels: mcLabelForRole("infra"),
+		},
+		Spec: mcfgv1.MachineConfigSpec{
+			Config: runtime.RawExtension{
+				Raw: helpers.MarshalOrDie(ctrlcommon.NewIgnConfig()),
+			},
+			Extensions: []string{"usbguard"},
+		},
 	}
 
+	_, err := cs.MachineConfigs().Create(context.TODO(), extensions, metav1.CreateOptions{})
+	require.Nil(t, err)
+	t.Logf("Created %s", extensions.Name)
+	renderedConfig, err := waitForRenderedConfig(t, cs, "infra", extensions.Name)
+	require.Nil(t, err)
+	if err := waitForPoolComplete(t, cs, "infra", renderedConfig); err != nil {
+		t.Fatal(err)
+	}
+	infraNode := getSingleNodeByRole(t, cs, "infra")
+
+	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
+	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+
+	installedExtesnions := execCmdOnNode(t, cs, infraNode, "chroot", "/rootfs", "rpm", "-qa", "usbguard")
+	if !strings.Contains(installedExtesnions, "usbguard") {
+		t.Fatalf("Node %s doesn't have expected extensions", infraNode.Name)
+	}
+	t.Logf("Node %s has expected extensions installed", infraNode.Name)
+
+	// Delete the applied kerneltype MachineConfig to make sure rollback works fine
+	if err := cs.MachineConfigs().Delete(context.TODO(), extensions.Name, metav1.DeleteOptions{}); err != nil {
+		t.Error(err)
+	}
+
+	t.Logf("Deleted MachineConfig %s", extensions.Name)
+
+	// Wait for the mcp to rollback to previous config
+	if err := waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig); err != nil {
+		t.Fatal(err)
+	}
+
+	// Re-fetch the infra node for updated annotations
+	infraNode = getSingleNodeByRole(t, cs, "infra")
+
+	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], oldInfraRenderedConfig)
+	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+	installedExtesnions = execCmdOnNode(t, cs, infraNode, "chroot", "/rootfs", "rpm", "-qa", "usbguard")
+	if strings.Contains(installedExtesnions, "usbguard") {
+		t.Fatalf("Node %s did not rollback successfully", infraNode.Name)
+	}
+	t.Logf("Node %s has successfully rolled back", infraNode.Name)
+
+	unlabelFunc()
+
+	workerMCP, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
+	require.Nil(t, err)
+	if err := wait.Poll(2*time.Second, 5*time.Minute, func() (bool, error) {
+		node, err := cs.Nodes().Get(context.TODO(), infraNode.Name, metav1.GetOptions{})
+		require.Nil(t, err)
+		if node.Annotations[constants.DesiredMachineConfigAnnotationKey] != workerMCP.Spec.Configuration.Name {
+			return false, nil
+		}
+		return true, nil
+	}); err != nil {
+		t.Errorf("infra node hasn't moved back to worker config: %v", err)
+	}
+	err = waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig)
+	require.Nil(t, err)
 }
 
 func TestPoolDegradedOnFailToRender(t *testing.T) {
 	cs := framework.NewClientSet("")
 
-	mcadd := createMCToAddFile("add-a-file", "/etc/mytestconfs", "test", "root")
-	ignCfg, _, err := ign.Parse(mcadd.Spec.Config.Raw)
+	mcadd := createMCToAddFile("add-a-file", "/etc/mytestconfs", "test")
+	ignCfg, err := ctrlcommon.ParseAndConvertConfig(mcadd.Spec.Config.Raw)
 	require.Nil(t, err, "failed to parse ignition config")
 	ignCfg.Ignition.Version = "" // invalid, won't render
 	rawIgnCfg := helpers.MarshalOrDie(ignCfg)
@@ -348,15 +368,12 @@ func TestReconcileAfterBadMC(t *testing.T) {
 	cs := framework.NewClientSet("")
 
 	// create a MC that contains a valid ignition config but is not reconcilable
-	mcadd := createMCToAddFile("add-a-file", "/etc/mytestconfs", "test", "root")
-	ignCfg, _, err := ign.Parse(mcadd.Spec.Config.Raw)
+	mcadd := createMCToAddFile("add-a-file", "/etc/mytestconfs", "test")
+	ignCfg, err := ctrlcommon.ParseAndConvertConfig(mcadd.Spec.Config.Raw)
 	require.Nil(t, err, "failed to parse ignition config")
-	ignCfg.Networkd = igntypes.Networkd{
-		Units: []igntypes.Networkdunit{
-			{
-				Name:     "test.network",
-				Contents: "test contents",
-			},
+	ignCfg.Storage.Disks = []ign3types.Disk{
+		ign3types.Disk{
+			Device: "/one",
 		},
 	}
 	rawIgnCfg := helpers.MarshalOrDie(ignCfg)
@@ -447,12 +464,16 @@ func TestReconcileAfterBadMC(t *testing.T) {
 	}
 }
 
+// Test that deleting a MC that changes a file does not completely delete the file
+// entirely but rather restores it to its original state.
 func TestDontDeleteRPMFiles(t *testing.T) {
 	cs := framework.NewClientSet("")
 
-	mcHostFile := createMCToAddFile("modify-host-file", "/etc/motd", "mco-test", "root")
+	unlabelFunc := labelRandomNodeFromPool(t, cs, "worker", "node-role.kubernetes.io/infra")
 
-	workerOldMc := getMcName(t, cs, "worker")
+	oldInfraRenderedConfig := getMcName(t, cs, "infra")
+
+	mcHostFile := createMCToAddFileForRole("modify-host-file", "infra", "/etc/motd", "mco-test")
 
 	// create the dummy MC now
 	_, err := cs.MachineConfigs().Create(context.TODO(), mcHostFile, metav1.CreateOptions{})
@@ -460,13 +481,10 @@ func TestDontDeleteRPMFiles(t *testing.T) {
 		t.Errorf("failed to create machine config %v", err)
 	}
 
-	renderedConfig, err := waitForRenderedConfig(t, cs, "worker", mcHostFile.Name)
+	renderedConfig, err := waitForRenderedConfig(t, cs, "infra", mcHostFile.Name)
+	require.Nil(t, err)
+	err = waitForPoolComplete(t, cs, "infra", renderedConfig)
 	require.Nil(t, err)
-
-	// wait for the mcp to go back to previous config
-	if err := waitForPoolComplete(t, cs, "worker", renderedConfig); err != nil {
-		t.Fatal(err)
-	}
 
 	// now delete the bad MC and watch the nodes reconciling as expected
 	if err := cs.MachineConfigs().Delete(context.TODO(), mcHostFile.Name, metav1.DeleteOptions{}); err != nil {
@@ -474,35 +492,69 @@ func TestDontDeleteRPMFiles(t *testing.T) {
 	}
 
 	// wait for the mcp to go back to previous config
-	if err := waitForPoolComplete(t, cs, "worker", workerOldMc); err != nil {
-		t.Fatal(err)
-	}
-
-	nodes, err := getNodesByRole(cs, "worker")
+	err = waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig)
 	require.Nil(t, err)
 
-	for _, node := range nodes {
-		assert.Equal(t, node.Annotations[constants.CurrentMachineConfigAnnotationKey], workerOldMc)
-		assert.Equal(t, node.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+	infraNode := getSingleNodeByRole(t, cs, "infra")
 
-		found := execCmdOnNode(t, cs, node, "cat", "/rootfs/etc/motd")
-		if strings.Contains(found, "mco-test") {
-			t.Fatalf("updated file doesn't contain expected data, got %s", found)
-		}
+	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], oldInfraRenderedConfig)
+	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+
+	found := execCmdOnNode(t, cs, infraNode, "cat", "/rootfs/etc/motd")
+	if strings.Contains(found, "mco-test") {
+		t.Fatalf("updated file doesn't contain expected data, got %s", found)
 	}
+
+	unlabelFunc()
+
+	workerMCP, err := cs.MachineConfigPools().Get(context.TODO(), "worker", metav1.GetOptions{})
+	require.Nil(t, err)
+	if err := wait.Poll(2*time.Second, 5*time.Minute, func() (bool, error) {
+		node, err := cs.Nodes().Get(context.TODO(), infraNode.Name, metav1.GetOptions{})
+		require.Nil(t, err)
+		if node.Annotations[constants.DesiredMachineConfigAnnotationKey] != workerMCP.Spec.Configuration.Name {
+			return false, nil
+		}
+		return true, nil
+	}); err != nil {
+		t.Errorf("infra node hasn't moved back to worker config: %v", err)
+	}
+	err = waitForPoolComplete(t, cs, "infra", oldInfraRenderedConfig)
+	require.Nil(t, err)
+
 }
 
-func TestCustomPool(t *testing.T) {
+func TestIgn3Cfg(t *testing.T) {
 	cs := framework.NewClientSet("")
 
 	unlabelFunc := labelRandomNodeFromPool(t, cs, "worker", "node-role.kubernetes.io/infra")
 
-	createMCP(t, cs, "infra")
+	// create a dummy MC with an sshKey for user Core
+	mcName := fmt.Sprintf("99-ign3cfg-infra-%s", uuid.NewUUID())
+	mcadd := &mcfgv1.MachineConfig{}
+	mcadd.ObjectMeta = metav1.ObjectMeta{
+		Name:   mcName,
+		Labels: mcLabelForRole("infra"),
+	}
+	// create a new MC that adds a valid user & ssh key
+	testIgn3Config := ign3types.Config{}
+	tempUser := ign3types.PasswdUser{Name: "core", SSHAuthorizedKeys: []ign3types.SSHAuthorizedKey{"1234_test_ign3"}}
+	testIgn3Config.Passwd.Users = append(testIgn3Config.Passwd.Users, tempUser)
+	testIgn3Config.Ignition.Version = "3.1.0"
+	mode := 420
+	testfiledata := "data:,test-ign3-stuff"
+	tempFile := ign3types.File{Node: ign3types.Node{Path: "/etc/testfileconfig"},
+		FileEmbedded1: ign3types.FileEmbedded1{Contents: ign3types.Resource{Source: &testfiledata}, Mode: &mode}}
+	testIgn3Config.Storage.Files = append(testIgn3Config.Storage.Files, tempFile)
+	rawIgnConfig := helpers.MarshalOrDie(testIgn3Config)
+	mcadd.Spec.Config.Raw = rawIgnConfig
 
-	infraMC := createMCToAddFileForRole("infra-host-file", "infra", "/etc/mco-custom-pool", "mco-custom-pool", "root")
-	_, err := cs.MachineConfigs().Create(context.TODO(), infraMC, metav1.CreateOptions{})
-	require.Nil(t, err)
-	renderedConfig, err := waitForRenderedConfig(t, cs, "infra", infraMC.Name)
+	_, err := cs.MachineConfigs().Create(context.TODO(), mcadd, metav1.CreateOptions{})
+	require.Nil(t, err, "failed to create MC")
+	t.Logf("Created %s", mcadd.Name)
+
+	// grab the latest worker- MC
+	renderedConfig, err := waitForRenderedConfig(t, cs, "infra", mcadd.Name)
 	require.Nil(t, err)
 	err = waitForPoolComplete(t, cs, "infra", renderedConfig)
 	require.Nil(t, err)
@@ -511,11 +563,18 @@ func TestCustomPool(t *testing.T) {
 
 	assert.Equal(t, infraNode.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
 	assert.Equal(t, infraNode.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
-	out := execCmdOnNode(t, cs, infraNode, "cat", "/rootfs/etc/mco-custom-pool")
-	if !strings.Contains(out, "mco-custom-pool") {
-		t.Fatalf("Unexpected infra MC content on node %s: %s", infraNode.Name, out)
+
+	foundSSH := execCmdOnNode(t, cs, infraNode, "grep", "1234_test_ign3", "/rootfs/home/core/.ssh/authorized_keys")
+	if !strings.Contains(foundSSH, "1234_test_ign3") {
+		t.Fatalf("updated ssh keys not found in authorized_keys, got %s", foundSSH)
 	}
-	t.Logf("Node %s has expected infra MC content", infraNode.Name)
+	t.Logf("Node %s has SSH key", infraNode.Name)
+
+	foundFile := execCmdOnNode(t, cs, infraNode, "cat", "/rootfs/etc/testfileconfig")
+	if !strings.Contains(foundFile, "test-ign3-stuff") {
+		t.Fatalf("updated file doesn't contain expected data, got %s", foundFile)
+	}
+	t.Logf("Node %s has file", infraNode.Name)
 
 	unlabelFunc()
 
@@ -533,56 +592,5 @@ func TestCustomPool(t *testing.T) {
 	}
 	err = waitForPoolComplete(t, cs, "infra", renderedConfig)
 	require.Nil(t, err)
-}
 
-func TestIgn3Cfg(t *testing.T) {
-	cs := framework.NewClientSet("")
-
-	// create a dummy MC with an sshKey for user Core
-	mcName := fmt.Sprintf("99-ign3cfg-worker-%s", uuid.NewUUID())
-	mcadd := &mcfgv1.MachineConfig{}
-	mcadd.ObjectMeta = metav1.ObjectMeta{
-		Name:   mcName,
-		Labels: mcLabelForWorkers(),
-	}
-	// create a new MC that adds a valid user & ssh key
-	testIgn3Config := ign3types.Config{}
-	tempUser := ign3types.PasswdUser{Name: "core", SSHAuthorizedKeys: []ign3types.SSHAuthorizedKey{"1234_test_ign3"}}
-	testIgn3Config.Passwd.Users = append(testIgn3Config.Passwd.Users, tempUser)
-	testIgn3Config.Ignition.Version = "3.0.0"
-	mode := 420
-	testfiledata := "data:,test-ign3-stuff"
-	tempFile := ign3types.File{Node: ign3types.Node{Path: "/etc/testfileconfig"},
-		FileEmbedded1: ign3types.FileEmbedded1{Contents: ign3types.FileContents{Source: &testfiledata}, Mode: &mode}}
-	testIgn3Config.Storage.Files = append(testIgn3Config.Storage.Files, tempFile)
-	rawIgnConfig := helpers.MarshalOrDie(testIgn3Config)
-	mcadd.Spec.Config.Raw = rawIgnConfig
-
-	_, err := cs.MachineConfigs().Create(context.TODO(), mcadd, metav1.CreateOptions{})
-	require.Nil(t, err, "failed to create MC")
-	t.Logf("Created %s", mcadd.Name)
-
-	// grab the latest worker- MC
-	renderedConfig, err := waitForRenderedConfig(t, cs, "worker", mcadd.Name)
-	require.Nil(t, err)
-	err = waitForPoolComplete(t, cs, "worker", renderedConfig)
-	require.Nil(t, err)
-	nodes, err := getNodesByRole(cs, "worker")
-	require.Nil(t, err)
-	for _, node := range nodes {
-		assert.Equal(t, node.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
-		assert.Equal(t, node.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
-
-		foundSSH := execCmdOnNode(t, cs, node, "grep", "1234_test_ign3", "/rootfs/home/core/.ssh/authorized_keys")
-		if !strings.Contains(foundSSH, "1234_test_ign3") {
-			t.Fatalf("updated ssh keys not found in authorized_keys, got %s", foundSSH)
-		}
-		t.Logf("Node %s has SSH key", node.Name)
-
-		foundFile := execCmdOnNode(t, cs, node, "cat", "/rootfs/etc/testfileconfig")
-		if !strings.Contains(foundFile, "test-ign3-stuff") {
-			t.Fatalf("updated file doesn't contain expected data, got %s", foundFile)
-		}
-		t.Logf("Node %s has file", node.Name)
-	}
 }
